<?php

if (!defined('_PS_VERSION_')) {
    exit();
}

include_once dirname(__FILE__). "/config/config.php";

use phpseclib3\Crypt\PublicKeyLoader;
use phpseclib3\Crypt\RSA;

class PaypayCard extends PaymentModule
{
    private $translationCode = [];
    private $RSAKey;

    /**
     * [__construct     first method to be called when the module is loaded by PrestaShop]
     */
    public function __construct()
    {
        $this->name = 'paypayCard';
        $this->tab = 'payments_gateways';
        $this->version = '2.1.9';
        $this->author = 'Acin iCloud Solutions';
        $this->bootstrap = true;
        $this->need_instance = 0;
        $this->ps_versions_compliancy = array('min' => '1.7.7.0', 'max' => '8.1.99');
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        parent::__construct();

        $this->confirmUninstall = $this->l('Tem certeza que deseja remover este módulo?');
        $this->displayName = $this->l('Cartão de crédito/débito (PayPay)');
        $this->description = $this->l('Solução de pagamentos');

        $this->initiateMissingTranslations();
        $this->getRSAKey();
    }

    private function getRSAKey()
    {
        $directory = _PS_UPLOAD_DIR_ . $this->name . "/RSAKey.pem";

        if (file_exists($directory)) {
            $this->RSAKey = PublicKeyLoader::load(file_get_contents($directory));
        } else {
            $this->RSAKey = RSA::createKey();
            mkdir(_PS_UPLOAD_DIR_ . $this->name);
            file_put_contents($directory, $this->RSAKey);
        }
    }

    private function removeDir()
    {
        if (unlink(_PS_UPLOAD_DIR_ . $this->name . "/RSAKey.pem")) {
            return rmdir(_PS_UPLOAD_DIR_ . $this->name);
        }
        return false;
    }

    private function initiateMissingTranslations()
    {
        $this->translationCode = [
            'CC' => $this->l('Cartão de crédito/débito (PayPay)'),
        ];
    }

    /**
     * [install     PrestaShop method call on shop administrator install the module]
     * @return [boolean] [success of installation]
     */
    public function install()
    {
        //check if Multistore features is enabled, and install current context to all shops
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        //Register hooks and clear configuration values
        if (version_compare(Tools::substr(_PS_VERSION_, 0, 3), '1.7', '>=')) {
            if (! parent::install() ||
             ! $this->registerHook('paymentOptions') ||
             ! $this->registerHook('displayHeader') ||
             ! $this->registerHook('displayOrderDetail') ||
             ! $this->registerHook('paymentReturn') ||
             ! $this->createTable() ||
             ! $this->createStates() ||
             ! $this->copyMails() ||
             ! $this->copyTPL() ||
             //clean PayPay configuration values from database
             ! Configuration::updateValue('PAYPAY_ENCRYPTION_KEY_CARD', '') ||
             ! Configuration::updateValue('PAYPAY_PLATAFORM_CODE_CARD', '') ||
             ! Configuration::updateValue('PAYPAY_NIF_CARD', '') ||
             ! Configuration::updateValue('WEBSERVICEURL_CARD', '') ||
             ! Configuration::updateValue('WEBSERVICEWSDL_CARD', '') ||
             ! Configuration::updateValue('HASH_CARD', '')
            ) {
                return false;
            }
        }
        return true;
    }

    /**
     * [uninstall   PrestaShop method on shop administrator uninstall the module]
     * @return [boolean] [success of uninstall]
     */
    public function uninstall()
    {
        if (! parent::uninstall() ||
            //Delete PayPay configuration values from database
             ! Configuration::deleteByName('PAYPAY_ENCRYPTION_KEY_CARD') ||
             ! Configuration::deleteByName('PAYPAY_PLATAFORM_CODE_CARD') ||
             ! Configuration::deleteByName('PAYPAY_NIF_CARD') ||
             ! Configuration::deleteByName('WEBSERVICEURL_CARD') ||
             ! Configuration::deleteByName('WEBSERVICEWSDL_CARD') ||
             ! Configuration::deleteByName('HASH_CARD') ||
             ! $this->removeDir()
        ) {
            return false;
        }
        return true;
    }

    /**
     * [createTable create table to storage PayPay payments]
     * @return [boolean] [create table success]
     */
    private function createTable()
    {
        $sql = sprintf(
            'CREATE TABLE IF NOT EXISTS `%s` (
                `id` INT(100) NOT NULL AUTO_INCREMENT,
                `idPaymentPaypay` INT(64) NOT NULL,
                `idOrder` INT(64) NOT NULL,
                `entity` VARCHAR(15),
                `amount` VARCHAR(15),
                `reference` VARCHAR(15),
                `possibleCreditCard` TINYINT(1) NOT NULL,
                `possibleBankPayment` TINYINT(1) NOT NULL,
                `paypayLink` VARCHAR(5000) NOT NULL,
                `redunicreLink` VARCHAR(5000),
                `hash` VARCHAR(50),
                `token` VARCHAR(50),
            PRIMARY KEY (id)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;',
            _DB_PREFIX_ . 'paypay_payments'
        );

        if (!Db::getInstance()->execute($sql)) {
            return false;
        }

        return true;
    }

    /**
     * [deleteTable delete table to storage PayPay payments]
     * @return [boolean] [delete table success]
     */
    private function deleteTable()
    {
        $sql = sprintf('DROP TABLE IF EXISTS `%s`;', _DB_PREFIX_ . 'paypay_payments');
        if (!Db::getInstance()->execute($sql)) {
            return false;
        }

        return true;
    }

    /**
     * getOrderDetails Function to retrieve the payment information from the DB
     * @author Fábio Abreu
     * @param  int $orderId id of the order that we will get the data
     * @return object          data related to the $orderId
     */
    private function getOrderDetails($orderId)
    {
        $sql = sprintf('SELECT *
                        FROM `%s`
                        WHERE idOrder=%u;', _DB_PREFIX_.'paypay_payments', $orderId);
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getOrderPaymentState Function to retrieve the payment state information from the DB
     * @author Fábio Abreu
     * @param  int $orderId id of the order that we will get the state
     * @return object          data with payment state of the order
     */
    private function getOrderPaymentState($orderId)
    {
        $sql = sprintf('SELECT *
                        FROM `%s`
                        WHERE id_order=%u;', _DB_PREFIX_.'orders', $orderId);
        return Db::getInstance()->executeS($sql);
    }

    /**
     * hookDisplayOrderDetail PrestaShop method called when user acess the order details
     * @author Fábio Abreu
     * @return tpl view with order details
     */
    public function hookDisplayOrderDetail($params)
    {
        $orderInfo=$this->getOrderDetails($params['order']->id);
        $orderPaymentState=$this->getOrderPaymentState($params['order']->id);
        $paymentMethod = $orderPaymentState['0']['payment'];

        if ($paymentMethod == "paypay_card") {
            $this->context->smarty->assign(
                array(
                    'logo'                      => __PS_BASE_URI__ . 'modules/'.$this->name.'/logo.png',
                    'cartLogo'                  => __PS_BASE_URI__ . 'modules/'.$this->name.'/views/img/carrinho.png',
                    'attentionLogo'             => __PS_BASE_URI__ . 'modules/'.$this->name.'/views/img/atencao.png',
                    'total'                     => Tools::displayPrice(
                        $params['order']->getOrdersTotalPaid(),
                        new Currency($params['order']->id_currency),
                        false
                    ),
                    'paypayLink'                => $orderInfo['0']['paypayLink'],
                    'paymentState'              => $orderPaymentState['0']['current_state'],
                    'paymentMethod'             => $paymentMethod,
                    'isPending'                 => $orderPaymentState['0']['current_state'] == Configuration::get('PAYPAY_STATUS_0')
                )
            );
            return $this->display(__FILE__, 'paypayHistoryInfo.tpl');
        }
    }

    /**
     * [hookDisplayPayment PrestaShop method call on user choice pay with PayPay]
     * @return [tpl]         [view with order details and order confirmation]
     */
    public function hookDisplayPayment()
    {
        $paymentOptionsList = $this->preparePayment();

        foreach ($paymentOptionsList as $option) {
            $option->link = $this->context->link->getModuleLink(
                $this->name,
                'communicatePayment',
                array(
                    'paypay_method' => array(
                        'code' => $option->code,
                        'name' => $option->name
                    )
                )
            );
        }

        $this->context->smarty->assign(array('paypayEncryptionKey' => $this->getEncriptionKey(),
                                             'paypayPlataformCode' => Configuration::get('PAYPAY_PLATAFORM_CODE_CARD'),
                                             'paypayNIF' => Configuration::get('PAYPAY_NIF_CARD'),
                                             'logo' => __PS_BASE_URI__ . 'modules/'.$this->name.'/logo.png',
                                             'actionUrl' => $this->context->link->getModuleLink($this->name, 'communicatePayment'),
                                             'isEURO' => $this->context->currency->name,
                                             'paymentOptionsList' => $paymentOptionsList
                                      ));

        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentOptions()
    {
        if (!$this->active) {
            return;
        }

        $payment_options = array();

        $isPaymentStep = Context::getContext()->controller->getCheckoutProcess()->getCurrentStep()->getIdentifier() === "checkout-payment-step";

        if (!$isPaymentStep) {
            return $payment_options;
        }

        $paymentOptionsList = $this->preparePayment();

        foreach ($paymentOptionsList as $key => $option) {
            $newOption = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
            $name = $this->translationCode[$option->code] ?? $this->l($option->name);

            $newOption->setCallToActionText($name)
                            ->setAction($this->context->link->getModuleLink($this->name, 'communicatePayment', array('paypay_method' => $option)))
                            ->setAdditionalInformation(
                                "<section>
                                    <p>".$this->l($option->description)."</p>
                                    </section>"
                            )
                            ->setLogo($option->iconUrl)
                            ->setModuleName($this->name);
            array_push($payment_options, $newOption);
        }

        return $payment_options;
    }

    /**
     * [hookDisplayHeader PrestaShop method add css in the header pages]
     * @return [css] css rules
     */
    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS(($this->_path).'/views/css/paypay.css', 'all');
    }

    /**
    * Function to get the payment options defined on PayPay's backoffice
    * @return array payment options
    */
    private function preparePayment()
    {
        $amount = Context::getContext()->cart->getordertotal();
        $amount = number_format($amount, 2, '', '');

        $lang = Context::getContext()->language->iso_code;

        $PayPayEncryptionKey = $this->getEncriptionKey();
        $PayPayPlataformCode = Configuration::get('PAYPAY_PLATAFORM_CODE_CARD');
        $PayPayNIF           = Configuration::get('PAYPAY_NIF_CARD');
        $WEBSERVICEURL       = Configuration::get('WEBSERVICEURL_CARD');
        $WEBSERVICEWSDL      = Configuration::get('WEBSERVICEWSDL_CARD');

        $classmap = array(
            'RequestEntity'                     => 'CardRequestEntity',
            'ResponseIntegrationState'          => 'CardResponseIntegrationState',
            'RequestInterval'                   => 'CardRequestInterval',
            'RequestPaymentReference'           => 'CardRequestPaymentReference',
            'ResponseEntityPayments'            => 'CardResponseEntityPayments',
            'ResponseEntityPaymentReferences'   => 'CardResponseEntityPaymentReferences',
            'RequestCreditCardPayment'          => 'CardRequestCreditCardPayment',
            'ResponseCreditCardPayment'         => 'CardResponseCreditCardPayment',
            'RequestReferenceDetails'           => 'CardRequestReferenceDetails',
            'ResponseGetPayment'                => 'CardResponseGetPayment',
            'RequestPaymentOrder'               => 'CardRequestPaymentOrder',
            'RequestPaymentOption'              => 'CardRequestPaymentOption',
            'ResponseWebhook'                   => 'CardResponseWebhook',
            'RequestWebhook'                    => 'CardRequestWebhook'
        );

        $options = array(
            'classmap' => $classmap,
            'location' => $WEBSERVICEURL,
            'cache_wsdl' => WSDL_CACHE_NONE
        );

        try {
            $dateTime                 = new DateTime();
            $dateTime                 = $dateTime->modify('+1 minutes');
            $dateTime                 = $dateTime->format('d-m-Y H:i:s');

            $client  = new SoapClient($WEBSERVICEWSDL, $options);
            $hash = hash("sha256", $PayPayEncryptionKey.$dateTime);
            $info = array(
                "Prestashop" => constant('_PS_VERSION_')
            );
            $entity  = new CardRequestEntity($PayPayPlataformCode, $hash, $dateTime, $PayPayNIF, $lang, $info);

            $array = array('amount'=>(int)$amount);
            $order = new CardRequestReferenceDetails($array);
            $order->paymentOptions = array(new CardRequestPaymentOption(array('code' => 'CC', 'type' => 'DEFAULT')));

            $paymentOptions = $client->validatePaymentReference($entity, $order);
            return $paymentOptions->paymentOptions;
        } catch (SoapFault $error) {
            $error = 0;
            return $error;
        }
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }

        $state = $params['order']->getCurrentState();

        if ($state == Configuration::get('PAYPAY_STATUS_0')) {
            $status = 'pendente';
        } elseif ($state == 2 || $state == Configuration::get('PAYPAY_STATUS_1')) {
            $status = 'pago';
        } elseif ($state == 6) {
            $status = 'cancelado';
        } else {
            $status = 'failed';
        }

        $this->smarty->assign('status', $status);

        return $this->fetch('module:'.$this->name.'/views/templates/hook/paymentReturnPage.tpl');
    }

    /**
     * [webHook send to PayPay webservice a url to receive a post data when orders as paid]
     * @return [boolean] [succes of webhook register]
     */
    private function webHook()
    {
        $PayPayEncryptionKey = $this->getEncriptionKey();
        $PayPayPlataformCode = Configuration::get('PAYPAY_PLATAFORM_CODE_CARD');
        $PayPayNIF           = Configuration::get('PAYPAY_NIF_CARD');
        $WEBSERVICEURL       = Configuration::get('WEBSERVICEURL_CARD');
        $WEBSERVICEWSDL      = Configuration::get('WEBSERVICEWSDL_CARD');

        $classmap = array(
            'RequestEntity'                     => 'CardRequestEntity',
            'ResponseIntegrationState'          => 'CardResponseIntegrationState',
            'RequestInterval'                   => 'CardRequestInterval',
            'RequestPaymentReference'           => 'CardRequestPaymentReference',
            'ResponseEntityPayments'            => 'CardResponseEntityPayments',
            'ResponseEntityPaymentReferences'   => 'CardResponseEntityPaymentReferences',
            'RequestCreditCardPayment'          => 'CardRequestCreditCardPayment',
            'ResponseCreditCardPayment'         => 'CardResponseCreditCardPayment',
            'RequestReferenceDetails'           => 'CardRequestReferenceDetails',
            'ResponseGetPayment'                => 'CardResponseGetPayment',
            'RequestPaymentOrder'               => 'CardRequestPaymentOrder',
            'RequestPaymentOption'              => 'CardRequestPaymentOption',
            'ResponseWebhook'                   => 'CardResponseWebhook',
            'RequestWebhook'                    => 'CardRequestWebhook'
        );

        $options = array(
            'classmap' => $classmap,
            'location' => $WEBSERVICEURL,
            'cache_wsdl' => WSDL_CACHE_NONE
        );

        try {
            $dateTime                 = new DateTime();
            $dateTime                 = $dateTime->modify('+1 minutes');
            $dateTime                 = $dateTime->format('d-m-Y H:i:s');

            $client  = new SoapClient($WEBSERVICEWSDL, $options);
            $hash = hash("sha256", $PayPayEncryptionKey.$dateTime);
            $info = array(
                "Prestashop" => constant('_PS_VERSION_')
            );
            $entity  = new CardRequestEntity($PayPayPlataformCode, $hash, $dateTime, $PayPayNIF, "PT", $info);

            $executionUrlWebhook      = $this->context->link->getModuleLink($this->name, 'updatePaymentWebhook');
            $requestWebhook           = new CardRequestWebhook(array('action'=>WEBHOOKACTION, 'url'=>$executionUrlWebhook));
            $result                   = $client->subscribeToWebhook($entity, $requestWebhook);

            if ($result->integrationState->state) {
                return true;
            } else {
                return false;
            }
        } catch (SoapFault $error) {
            $error = 0;
            return $error;
        }
    }

    /**
     * [getContent      PrestaShop method triggers on installation module is completed,
     * and show the module configuration page]
     * @return [type] [description]
     */
    public function getContent()
    {
        $output = null;

        //on clicked in submit button
        if (Tools::isSubmit('submit'.$this->name)) {
            if ($this->saveConfigurationData()) {
                $output .= $this->displayConfirmation($this->l('Configurações guardadas com sucesso'));
            } else {
                $output .= $this->displayError($this->l('Erro ao guardar configurações'));
            }
        }

        $output .= $this->displayInfo();
        return $output .= $this->displayForm();
    }

    /**
     * [displayInfo   load view to update manual payment status]
     * @return [tpl]
     */
    private function displayInfo()
    {
        $this->context->smarty->assign(
            array(
                'urlUpdateStatePayments' => $this->context->link->getModuleLink($this->name, 'updateStatePayments')
                                            .'?hash='.Configuration::get('HASH_CARD'),
                'notice' => Module::isInstalled('Paypay') & Module::isEnabled('Paypay'),
                'soap'   => !class_exists('SoapClient'),
            )
        );
        return $this->display(__FILE__, 'views/templates/admin/backOffice.tpl');
    }

    /**
     * [displayForm    PrestaShop method to construct and show configuration page, usng a PrestaShop form helper]
     * @return [form]
     */
    public function displayForm()
    {
        //Get default language
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        //Init fields form array
        $fields_form = array();
        $fields_form[0]['form'] = array('legend' => array('title' => $this->l('Configurações')),
                                        'input' => array( array('type'  => 'text',
                                                                'label' => $this->l('Chave de Encriptação'),
                                                                'name'  => 'paypayEncryptionKey',
                                                                'size'  => 20,
                                                                'required' => true),
                                                          array('type'  => 'text',
                                                                'label' => $this->l('Código da Plataforma'),
                                                                'name'  => 'paypayPlataformCode',
                                                                'size'  => 4,
                                                                'required' => true),
                                                          array('type'  => 'text',
                                                                'label' => $this->l('NIF'),
                                                                'name'  => 'paypayNIF',
                                                                'size'  => 9,
                                                                'required' => true),
                                                          array('type'  => 'radio',
                                                                'label'    => $this->l('Ambiente'),
                                                                'name'     => 'opEnv',
                                                                'required' => true,
                                                                'is_bool'  => true,
                                                         'values' => array( array('id'     => 'prod',
                                                                                   'value' => 0,
                                                                                   'label' => $this->l('Produção')),
                                                                            array('id'    => 'beta',
                                                                                  'value' => 1,
                                                                                  'label' => $this->l('Beta'))
                                                            )),
                                                    ),
                                        'submit' => array('title' => $this->l('Guardar'),
                                                          'class' => 'btn btn-default pull-right'
                                                    )
                                        );
        $helper = new HelperForm();

        //Module, Token and currentIndex
        $helper->module          = $this;
        $helper->name_controller = $this->name;
        $helper->token           = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex    = AdminController::$currentIndex.'&configure='.$this->name;

        //Language
        $helper->default_form_language    = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and submit action
        $helper->title          = $this->displayName;
        $helper->submit_action  = 'submit'.$this->name;

        //Load current confguration values
        $helper->fields_value['paypayEncryptionKey'] = $this->getEncriptionKey();
        $helper->fields_value['paypayPlataformCode'] = Configuration::get('PAYPAY_PLATAFORM_CODE_CARD');
        $helper->fields_value['paypayNIF'          ] = Configuration::get('PAYPAY_NIF_CARD');
        $helper->fields_value['opEnv'              ] = Configuration::get('PAYPAY_ENV_CARD');

        return $helper->generateForm($fields_form);
    }

    /**
     * [saveConfigurationData   save a configuration]
     * @return [boolean] [success of saved configuration]
     */
    private function saveConfigurationData()
    {
        if (Tools::isSubmit('submit'.$this->name)) {
            $result     = [];
            $updateData = array(
                'paypayEncryptionKey' => 'PAYPAY_ENCRYPTION_KEY_CARD',
                'paypayPlataformCode' => 'PAYPAY_PLATAFORM_CODE_CARD',
                'paypayNIF'           => 'PAYPAY_NIF_CARD',
            );

            foreach ($updateData as $postIndex => $configIndex) {
                if (Tools::getValue($postIndex)) {
                    $value = Tools::getValue($postIndex);

                    if ($postIndex === 'paypayEncryptionKey') {
                        $value = base64_encode($this->RSAKey->getPublicKey()->encrypt($value));
                    }

                    array_push($result, Configuration::updateValue($configIndex, $value));
                }
            }

            Configuration::updateValue('HASH_CARD', md5(date('Y-m-d H:i:s')));

            //Check have a error on update values in database
            if (in_array('0', $result)) {
                return false;
            }

            if (!defined('PAYPAY_SERVER_URL') && !defined('PAYPAY_WSDL_URL')) {
                if (Tools::getValue('opEnv') == '1') { //Beta
                    Configuration::updateValue('WEBSERVICEURL_CARD', WEBSERVICEURL_BETA);
                    Configuration::updateValue('WEBSERVICEWSDL_CARD', WEBSERVICEWSDL_BETA);
                    Configuration::updateValue('PAYPAY_ENV_CARD', 1);
                }

                if (Tools::getValue('opEnv') == '0') { //Production
                    Configuration::updateValue('WEBSERVICEURL_CARD', WEBSERVICEURL_PROD);
                    Configuration::updateValue('WEBSERVICEWSDL_CARD', WEBSERVICEWSDL_PROD);
                    Configuration::updateValue('PAYPAY_ENV_CARD', 0);
                }
            } else {
                Configuration::updateValue('WEBSERVICEURL_CARD', constant('PAYPAY_SERVER_URL'));
                Configuration::updateValue('WEBSERVICEWSDL_CARD', constant('PAYPAY_WSDL_URL'));
            }

            if ($this->webHook()) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * [getOrderState return a order status description by order state]
     * @param  [string] $id_order_state [order status id]
     * @return [string]                 [order status description]
     */
    public static function getOrderState($id_order_state)
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow(
            'SELECT `id_order_state` AS ok
            FROM `' . _DB_PREFIX_ . 'order_state`
            WHERE `id_order_state` = ' . (int) $id_order_state
        );
    }

    /**
     * createStates Create the payment states for PayPay module, we need to check if doens`t exists
     * @return boolean success states creation
     */
    public function createStates()
    {
        $order_states = array(
            array(
                '#ccfbff',
                $this->l('Aguardando Pagamento'),
                false,
                '0',
                '0'
            )
        );

        foreach ($order_states as $key => $value) {
            $order_state              = new OrderState();
            $order_state->invoice     = $value[3];
            $order_state->send_email  = false;
            $order_state->module_name = 'PayPay';
            $order_state->color       = $value[0];
            $order_state->hidden      = false;
            $order_state->logable     = $value[2];
            $order_state->delivery    = false;
            $order_state->paid        = $value[4];
            $order_state->name        = array();

            foreach (Language::getLanguages(false) as $language) {
                $order_state->name[(int) $language['id_lang']] = $value[1];
            }

            if (! $order_state->save()) {
                return false;
            }

            Configuration::updateValue('PAYPAY_STATUS_' . $key, $order_state->id);
        }
        return true;
    }

    /**
     * [copyMails copy emails to PrestaShop mail folder]
     * @return [boolean] [success copy mails]
     */
    private function copyMails()
    {
        $languages = array('pt','es','en');

        foreach ($languages as $language) {
            $html = _PS_MAIL_DIR_ .$language. '/info_order.html';
            $txt  = _PS_MAIL_DIR_ .$language. '/info_order.txt';
            $cancelHtml = _PS_MAIL_DIR_ .$language. '/order_canceled.html';
            $cancelTxt  = _PS_MAIL_DIR_ .$language. '/order_canceled.txt';

            if (! file_exists(_PS_MAIL_DIR_ . $language)) {
                mkdir(_PS_MAIL_DIR_ . $language, 0777, true);
            }

            $htmlToCopy = dirname(__file__) . '/mails/' .$language. '/info_order.html';
            copy($htmlToCopy, $html);

            $txtToCopy = dirname(__file__) . '/mails/' .$language. '/info_order.txt';
            copy($txtToCopy, $txt);

            $htmlToCopy = dirname(__file__) . '/mails/' .$language. '/order_canceled.html';
            copy($htmlToCopy, $cancelHtml);

            $txtToCopy = dirname(__file__) . '/mails/' .$language. '/order_canceled.txt';
            copy($txtToCopy, $cancelTxt);
        }
        return true;
    }

    /**
     * Open the order-detail.tpl of the theme and add the nofilter params to the hook
     * @author Fábio Abreu
     * @return bool return true if the function runs successfully
     */
    private function copyTPL()
    {
        $path_to_file = _PS_THEME_DIR_.'templates/customer/order-detail.tpl';

        if (!file_exists($path_to_file)) {
            mkdir($path_to_file, 0777, true);
        }

        $file_contents = Tools::file_get_contents($path_to_file);
        if (stripos($file_contents, "HOOK_DISPLAYORDERDETAIL nofilter") == false) {
            $file_contents = str_replace("HOOK_DISPLAYORDERDETAIL", "HOOK_DISPLAYORDERDETAIL nofilter", $file_contents);
            file_put_contents($path_to_file, $file_contents);
        }

        return true;
    }

    private function getEncriptionKey()
    {
        $encryptKey = Configuration::get('PAYPAY_ENCRYPTION_KEY_CARD');

        try {
            return $this->RSAKey->decrypt(base64_decode($encryptKey));
        } catch (LengthException $e) {
            if ($encryptKey) {
                Configuration::updateValue('PAYPAY_ENCRYPTION_KEY_CARD', base64_encode($this->RSAKey->getPublicKey()->encrypt($encryptKey)));
            }
            return $encryptKey;
        }
    }
}
